@@ -3,7 +3,7 @@ use strict;
 use warnings;
 no warnings 'redefine';
 
-our $VERSION = do { my @r = ( q$Revision: 2.10 $ =~ /\d+/g ); sprintf "%d." . "%02d" x $#r, @r };
+our $VERSION = do { my @r = ( q$Revision: 2.18 $ =~ /\d+/g ); sprintf "%d." . "%02d" x $#r, @r };
 use Encode qw(find_encoding encode_utf8 decode_utf8);
 use MIME::Base64;
 use Carp;
@@ -31,7 +31,7 @@ $Encode::Encoding{'MIME-Q'} = bless {
     Name     => 'MIME-Q',
 } => __PACKAGE__;
 
-use base qw(Encode::Encoding);
+use parent qw(Encode::Encoding);
 
 sub needs_lines { 1 }
 sub perlio_ok   { 0 }
@@ -40,14 +40,11 @@ sub decode($$;$) {
     use utf8;
     my ( $obj, $str, $chk ) = @_;
 
-    # zap spaces between encoded words
-    $str =~ s/\?=\s+=\?/\?==\?/gos;
-
     # multi-line header to single line
-    $str =~ s/(?:\r\n|[\r\n])[ \t]+//gos;
+    $str =~ s/(?:\r\n|[\r\n])[ \t]//gos;
 
     1 while ( $str =~
-        s/(=\?[-0-9A-Za-z_]+\?[Qq]\?)(.*?)\?=\1(.*?\?=)/$1$2$3/ )
+              s/(=\?[-0-9A-Za-z_]+\?[Qq]\?)([^?]*?)\?=\1([^?]*?\?=)/$1$2$3/ )
       ;    # Concat consecutive QP encoded mime headers
            # Fixes breaking inside multi-byte characters
 
@@ -95,7 +92,7 @@ sub decode_q {
 
 my $especials =
   join( '|' => map { quotemeta( chr($_) ) }
-      unpack( "C*", qq{()<>@,;:"'/[]?.=} ) );
+      unpack( "C*", qq{()<>,;:"'/[]?=} ) );
 
 my $re_encoded_word = qr{
     =\?                # begin encoded word
@@ -108,12 +105,17 @@ my $re_encoded_word = qr{
 
 my $re_especials = qr{$re_encoded_word|$especials}xo;
 
+# cf:
+#    https://rt.cpan.org/Ticket/Display.html?id=88717
+#    https://www.ietf.org/rfc/rfc0822.txt
+my $re_linear_white_space = qr{(?:[ \t]|\r\n?)};
+
 sub encode($$;$) {
     my ( $obj, $str, $chk ) = @_;
     my @line = ();
     for my $line ( split /\r\n|[\r\n]/o, $str ) {
         my ( @word, @subline );
-        for my $word ( split /($re_especials)/o, $line ) {
+        for my $word ( split /($re_linear_white_space+)/o, $line ) {
             if (   $word =~ /[^\x00-\x7f]/o
                 or $word =~ /^$re_encoded_word$/o )
             {
@@ -127,18 +129,19 @@ sub encode($$;$) {
         for my $word (@word) {
             use bytes ();
             if ( bytes::length($subline) + bytes::length($word) >
-                $obj->{bpl} )
+                $obj->{bpl} - 1 )
             {
                 push @subline, $subline;
                 $subline = '';
             }
+            $subline .= ' ' if ($subline =~ /\?=$/ and $word =~ /^=\?/);
             $subline .= $word;
         }
-        $subline and push @subline, $subline;
-        push @line, join( "\n " => @subline );
+        length($subline) and push @subline, $subline;
+        push @line, join( "\n " => grep !/^$/, @subline );
     }
     $_[1] = '' if $chk;
-    return join( "\n", @line );
+    return (substr($str, 0, 0) . join( "\n", @line ));
 }
 
 use constant HEAD   => '=?UTF-8?';
